package com.vcarecity.cvs.core;

import java.lang.reflect.Modifier;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;

/**
 * @author Kerry on 19/11/21
 */

public class AutoMapper {


    public static <T> T autoMapper(ResultSet resultSet, Class<T> cls) {
        List<JavaMapColumn> fieldColumnList = ReflectionUtil.getFieldColumnList(cls);
        try {
            final T t = cls.newInstance();
            for (JavaMapColumn fc : fieldColumnList) {
                if (Modifier.isPrivate(fc.getField().getModifiers())) {
                    fc.getField().setAccessible(true);
                }
                Class<?> type = fc.getField().getType();
                Object object;
                if (type == java.util.Date.class) {
                    Timestamp timestamp = resultSet.getTimestamp(fc.getColumn());
                    object = Date.from(timestamp.toInstant());
                } else if (type == LocalDateTime.class) {
                    Timestamp timestamp = resultSet.getTimestamp(fc.getColumn());
                    object = timestamp.toLocalDateTime();
                } else if (type == LocalDate.class) {
                    Timestamp timestamp = resultSet.getTimestamp(fc.getColumn());
                    object = timestamp.toLocalDateTime().toLocalDate();
                } else if (type == LocalTime.class) {
                    Timestamp timestamp = resultSet.getTimestamp(fc.getColumn());
                    object = timestamp.toLocalDateTime().toLocalTime();
                } else {
                    // Timestamp 时出现 HOUR_OF_DAY 2->3 异常.请在mysql url 中加入 &serverTimezone=Asia/Shanghai
                    object = resultSet.getObject(fc.getColumn(), type);
                }
                fc.getField().set(t, object);

            }
            return t;
        } catch (InstantiationException | IllegalAccessException | SQLException e) {
            e.printStackTrace();
        }
        return null;
    }


    public static <T> Object[] autoMapper(T t, Class<T> cls) {
        List<JavaMapColumn> fieldColumnList = ReflectionUtil.getFieldColumnList(cls);
        Object[] contents = new Object[fieldColumnList.size()];
        try {
            for (int i = 0; i < fieldColumnList.size(); i++) {
                final JavaMapColumn fc = fieldColumnList.get(i);
                if (Modifier.isPrivate(fc.getField().getModifiers())) {
                    fc.getField().setAccessible(true);
                }
                contents[i] = fc.getField().get(t);
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return contents;
    }
}
