package com.vcarecity.cvs.core;

import com.vcarecity.cvs.annotation.Tables;
import org.reflections.Reflections;

import javax.persistence.Column;
import javax.persistence.Table;
import javax.persistence.Transient;
import java.lang.reflect.Field;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author Kerry on 19/11/21
 */

public class ReflectionUtil {


    private static final Map<Class<?>, List<JavaMapColumn>> FIELD_CACHE = new ConcurrentHashMap<>();


    public static List<JavaMapColumn> getFieldColumnList(Class<?> cls) {

        List<JavaMapColumn> fields = FIELD_CACHE.get(cls);
        if (fields != null) {
            return fields;
        }
        synchronized (FIELD_CACHE) {
            Field[] declaredFields = cls.getDeclaredFields();
            fields = new ArrayList<>(declaredFields.length);

            for (Field declaredField : declaredFields) {
                Transient ta = declaredField.getAnnotation(Transient.class);
                if (ta != null) {
                    continue;
                }
                final Column column = declaredField.getAnnotation(Column.class);
                String fieldName = declaredField.getName();
                if (column != null) {
                    final String name = column.name();
                    if (name.length() > 0) {
                        fieldName = name;
                    }
                }
                fields.add(new JavaMapColumn(declaredField, fieldName));
            }
            FIELD_CACHE.put(cls, fields);
            return FIELD_CACHE.get(cls);
        }
    }

    private static Map<String, Class<?>> classMap = new HashMap<>();

    public static void initTableClassMapper(String... pkgs) {

        Map<String, Class<?>> tableMap = new HashMap<>();
        Map<String, Class<?>> tablesMap = new HashMap<>();

        for (String pkg : pkgs) {
            Reflections reflections = new Reflections(pkg);

            Set<Class<?>> classSet = reflections.getTypesAnnotatedWith(Table.class);
            for (Class<?> cls : classSet) {
                final Table table = cls.getAnnotation(Table.class);
                final String name = table.name();
                if (tableMap.containsKey(name)) {
                    throw new RuntimeException("tableName= " + name + " has more Class.[" + tableMap.get(name) + "," + cls + "]");
                }
                tableMap.put(name, cls);
            }

            Set<Class<?>> tablesSet = reflections.getTypesAnnotatedWith(Tables.class);
            for (Class<?> cls : tablesSet) {
                final Tables tables = cls.getAnnotation(Tables.class);
                final String[] value = tables.value();
                for (String name : value) {
                    if (tablesMap.containsKey(name)) {
                        throw new RuntimeException("tableName= " + name + " has more Class.[" + tablesMap.get(name) + "," + cls + "]");
                    }
                    tablesMap.put(name, cls);
                }
            }
        }

        classMap.putAll(tableMap);
        classMap.putAll(tablesMap);

    }


    @SuppressWarnings("unchecked")
    public static <T> Class<T> getTableClass(String table) {
        return (Class<T>) classMap.get(table);
    }

}
