package com.vcarecity.cvs.module;

import com.google.inject.AbstractModule;
import com.google.inject.Inject;
import com.google.inject.Provides;
import com.google.inject.Singleton;
import com.google.inject.assistedinject.FactoryModuleBuilder;
import com.vcarecity.cvs.event.ExportEventListener;
import com.vcarecity.cvs.event.MySQLImportEvent;
import com.vcarecity.cvs.factory.ResultHandlerFactory;
import com.vcarecity.cvs.properties.AppProperties;
import com.vcarecity.cvs.properties.DbProperties;
import com.vcarecity.cvs.service.ResultHandlerService;
import com.vcarecity.cvs.service.SQLQueryService;
import com.vcarecity.cvs.service.impl.*;
import com.vcarecity.cvs.type.ResultHandlerEnum;
import com.vcarecity.cvs.util.CreateDatasource;
import com.zaxxer.hikari.HikariDataSource;

/**
 * @author Kerry on 19/11/20
 */

public class SQLModule extends AbstractModule {
    private final ResultHandlerEnum resultHandlerEnum;

    public SQLModule(ResultHandlerEnum resultHandlerEnum) {
        this.resultHandlerEnum = resultHandlerEnum;
    }

    @Override
    protected void configure() {

        bind(ExportEventListener.class).to(MySQLImportEvent.class);

        Class<? extends ResultHandlerService> resultClass;

        switch (resultHandlerEnum) {
            case CSV:
                resultClass = CSVResultHandlerServiceImpl.class;
                break;
            case MYSQL:
                resultClass = MySQLResultStatHandlerServiceImpl.class;
                break;
            case MYSQL_PS:
                resultClass = MySQLResultHandlerServiceImpl.class;
                break;
            case DEFAULT:
            default:
                resultClass = LogResultHandlerServiceImpl.class;
                break;
        }

        install(new FactoryModuleBuilder()
                .implement(ResultHandlerService.class, resultClass)
                .build(ResultHandlerFactory.class));

    }


    @Inject
    @Provides
    public SQLQueryService sqlQueryService(HikariDataSource dataSource) {
        final String url = dataSource.getJdbcUrl();
        if (url.startsWith("jdbc:mysql")) {
            return new MySQLQueryServiceImpl(dataSource);
        } else if (url.startsWith("jdbc:oracle")) {
            return new OracleQueryServiceImpl(dataSource);
        }
        throw new Error("not support this jdbc driver class = " + dataSource.getDriverClassName());
    }

    @Provides
    @Singleton
    @Inject
    public HikariDataSource dataSource(AppProperties properties) {
        // @see https://github.com/brettwooldridge/HikariCP/wiki/MySQL-Configuration

        final DbProperties database = properties.getFromDb();
        final HikariDataSource hikariDataSource = CreateDatasource.createDataSource(database);
        Runtime.getRuntime().addShutdownHook(new Thread(hikariDataSource::close));
        return hikariDataSource;
    }

}
