package com.vcarecity.cvs.shell;

import com.vcarecity.cvs.properties.DbProperties;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 * @author Kerry on 19/11/29
 */

@Slf4j
public class MySQLImporter implements Runnable {

    private final DbProperties dbProperties;

    private final String filepath;
    private final String table;
    private final String[] header;

    private static final String LOAD_DATA_SQL_TEP = "LOAD DATA INFILE '%s' " +
            "INTO TABLE %s " +
            "FIELDS " +
            "TERMINATED BY '\\t' " +
            "ENCLOSED BY '\\\"' " +
            "ESCAPED BY '\\\\' " +
            "LINES TERMINATED BY '\\n' (%s)";

    public MySQLImporter(DbProperties dbProperties, String filepath, String table, String[] header) {
        this.dbProperties = dbProperties;
        this.filepath = filepath;
        this.table = table;
        this.header = header;
    }

    @Override
    public void run() {
        File file = new File(filepath);
        if (!file.exists()) {
            return;
        }

        final String sql = String.format(LOAD_DATA_SQL_TEP, filepath, table, String.join(",", this.header));

        logger.info("sql = {}", sql);
        try {
            importData(sql);
        } catch (SQLException e) {
            e.printStackTrace();
        }
    }

    private void execShell(String sql) {
        //Runtime.getRuntime().exec()
    }


    private void importFromLocal(String sql) {
        //if (statement.isWrapperFor(JdbcStatement.class)) {
        //    final JdbcStatement unwrap = statement.unwrap(JdbcStatement.class);
        //    unwrap.setLocalInfileInputStream();
        //}
    }


    private void importData(String sql) throws SQLException {
        try (final Connection connection = getConnection();
             final PreparedStatement statement = connection.prepareStatement(sql)) {
            statement.executeUpdate();
        }
    }

    private Connection getConnection() throws SQLException {
        return DriverManager.getConnection(dbProperties.getUrl(), dbProperties.getUser(), dbProperties.getPassword());
    }

}
